﻿import React from 'react';
import PanelItem from './panel-item';
import StyleSheetBuilder from "./style-sheet-builder";
import './panel.css';

export default class HorizontalPanel extends React.Component {
    constructor(props) {
        super(props);

        this.state = {
        };

        this.styleElement = null;

        this.isVertical = this.props.isVertical == true ? true : false;
        this.spacing = this.props.spacing ? this.props.spacing : 5;
        this.responsive = this.props.responsive == false? false: true;
        this.responsiveWidth = this.props.responsiveWidth ? this.props.responsiveWidth : 700;

        this.id = "horizontal-panel-" + HorizontalPanelIdIncrementor.create();
    }

    componentDidMount() {
        if (this.responsive == true) {
            var style = new StyleSheetBuilder();
            var styleFirst = new StyleSheetBuilder();

            style.add("width", "auto !important");
            style.add("display", "block !important");
            style.add("float", "none !important");
            style.add("margin-left", "0 !important");
            style.add("margin-right", "0 !important");
            style.add("padding-left", "0 !important");
            style.add("padding-right", "0 !important");
            style.add("padding-top", this.spacing + "px !important");

            styleFirst.add("padding-top", "0 !important");

            var cssText = "@media all and (max-width: " + this.responsiveWidth + "px) {" +
                ".panel-component[data-panel-component-id=" + this.id + "] " + ">" + " .panel-item-component[data-panel-item-component-id] {" +
                style.toString() +
                "}" +
                ".panel-component[data-panel-component-id=" + this.id + "] " + ">" + " .panel-item-component[data-panel-item-component-id]:first-child {" +
                styleFirst.toString() +
                "}" +
                "}";

            this.styleElement.innerText = cssText;
        }
    }

    render() {
        var pannelStyle = {};

        if(!this.isVertical) {
            //pannelStyle.whiteSpace = "nowrap";
        }

        if (Object.prototype.toString.call(this.props.children) === '[object Array]') {
            var children = this.props.children.filter(c => c);
        }
        else {
            var children = this.props.children;
        }

        var extendChildProps = (child, props, index) => {
            props.spacing = this.spacing;
            props.isVertical = this.isVertical;

            props.isChild = this.props.isChild;
            props.responsive = this.responsive;
            props.responsiveWidth = this.responsiveWidth;
        };

        var extendedChildren = children;
        var hasFloat = undefined;
        if (children && children.length > 1) {
            // if doesn't have fixed OR vertical
            if (children.filter(child => child.props.width > 0).length == 0 || this.isVertical == true) {
                var width = (100 / children.length) + "%";
                var display = "inline-block";

                extendedChildren = React.Children.map(children,
                    (child, index) => {
                        var props = {
                            width: width,
                            display: display,
                            isFirstHorizontal: index == 0,
                            isFirst: index == 0,
                            isLast: children.length == (index + 1),
                        };

                        extendChildProps(child, props, index);

                        return React.cloneElement(child, props);
                    });
            }
            else {
                var autoMarginLeft = 0;
                var autoMarginRight = 0;
                var autoItemIndex = undefined;

                // init marging
                var wrappers = React.Children.map(children,
                   (child, index) => {
                       var wrapper = {
                           width: child.props.width,
                           isAuto: undefined,
                           float: undefined,
                           index: index,
                       }
                       // auto
                       if (wrapper.width == undefined) {
                           wrapper.isAuto = true;
                           autoItemIndex = index;
                       }
                           // width
                       else {
                           // float - left
                           if (index < autoItemIndex || autoItemIndex == undefined) {
                               autoMarginLeft += wrapper.width;
                               wrapper.float = "left";
                           }
                               // float - right
                           else {
                               autoMarginRight += wrapper.width;
                               wrapper.float = "right";
                           }
                           hasFloat =true;
                       }

                       return wrapper;
                   });

                // move "auto" to last position
                if (autoItemIndex != undefined) {
                    var wrapper = wrappers[autoItemIndex];
                    wrappers.splice(autoItemIndex, 1);
                    wrappers.push(wrapper);
                }

                extendedChildren = wrappers.map((wrapper, index) => {
                    var child = children[wrapper.index];
                    var props = {
                        width: wrapper.width,
                        float: wrapper.float,
                        isFirstHorizontal: wrapper.index == 0,
                        isFirst: index == 0,
                        isLast: children.length == (index + 1),
                        key: index
                    };

                    if (wrapper.isAuto) {
                        props.marginLeft = autoMarginLeft;
                        props.marginRight = autoMarginRight;
                    }

                    extendChildProps(child, props, index);

                    return React.cloneElement(child, props)
                });
            }
        }
        return (
            <div className={"panel-component"} style={pannelStyle} data-panel-component-id={this.id}>{extendedChildren}
                {hasFloat && <div className="clear-fix"></div>}
                <style ref={x => this.styleElement=x}>
                </style>
            </div>);
    }
}

HorizontalPanel.propTypes = {
    children: function (props, propName, componentName) {
        const prop = props[propName]
        let error = null
        let fixedWidthCount = 0;
        let autoWidthCount = 0;

        React.Children.forEach(prop, function (child) {
            if (child ==  null) {
                return; // null instead of PanelItem allowed
            }
            if (child.type !== PanelItem) {
                error = new Error('' + componentName + ' children should be of type "PanelItem".');
            }
            if (child.width == undefined) {
                autoWidthCount++;
            }
            else {
                fixedWidthCount++;
            }
        });

        if (fixedWidthCount > 0) {
            if (autoWidthCount > 1) {
                error = new Error("If HorizontalPanel has at least one fixed-width PanelItem it should have only one auto-width PanelItem.");
            }
        }

        return error;
    }
}


if(typeof window !== "undefined" && window.HorizontalPanelIdIncrementor == null) {
    window.HorizontalPanelIdIncrementor = new function () {
        var self = this;
        self._id = 0;
        self.create = function () {
            self._id++;
            return self._id;
        };
    }
}
