﻿import React from 'react';
import HorizontalPanel from './horizontal-panel';
import VerticalPanel from './vertical-panel';
import StyleSheetBuilder from "./style-sheet-builder";

import './panel-item.css';

export default class PanelItem extends React.Component {
    constructor(props) {
        super(props);

        this.state = {
        };

        this.spacing = this.props.spacing ? this.props.spacing : 5;

        this.id = "panel-item-" + PanelItemIdIncrementor.create();
    }

    render() {
        var props = this.props;

        var style = new StyleSheetBuilder();

        if (props.isVertical == true) {
            if (this.spacing) {
                style.addPx("padding-top", this.spacing);
            }

            if (props.isFirst) {
                style.addPx("padding-top", 0);
            }
        }
        else {
            style.addPx("width", props.width);

            style.add("display", props.display);
            style.add("float", props.float);

            style.addPx("margin-left", props.marginLeft);
            style.addPx("margin-right", props.marginRight);

            if (this.spacing) {
                style.addPx("padding-top", props.spacing);
                style.addPx("padding-left", props.spacing);
                style.addPx("padding-bottom", props.spacing);
            }

            if (props.isFirstHorizontal) {
                style.paddingLeft = 0;
                style.addPx("padding-left", 0);
            }

            if (props.isChild) {
                style.addPx("padding-top", 0);
                style.addPx("padding-bottom", 0);
            }
        }

        var children = React.Children.map(this.props.children,
                    (child, index) => {
                        var element = child;
                        if (child && (child.type == HorizontalPanel
                            || child.type == VerticalPanel)) {

                            var extendProps = {
                                isChild: true,
                                spacing: this.spacing,
                            };

                            // pass responsive
                            if (child.props.responsive == undefined && this.props.responsive != undefined) {
                                extendProps.responsive = this.props.responsive;
                            }

                            // pass responsiveWidth
                            if (child.props.responsiveWidth == undefined && this.props.responsiveWidth != undefined) {
                                extendProps.responsiveWidth = this.props.responsiveWidth;
                            }
                            

                            element = React.cloneElement(child, extendProps);
                        }
                        return element;
                    });

        children = children ? children.filter(child => child) : null;

        return (
            <div className="panel-item-component" data-panel-item-component-id={this.id}>{children}
                <style>
                    {
                        " .panel-item-component[data-panel-item-component-id=" + this.id + "] { "+ style.toString() +" }"
                    }
                </style>
            </div>);
    }
}

if(typeof window !== "undefined" && window.PanelItemIdIncrementor == null) {
    window.PanelItemIdIncrementor = new function () {
        var self = this;
        self._id = 0;
        self.create = function () {
            self._id++;
            return self._id;
        }
    }
}

//export { PanelItem }